#!/usr/bin/env python


# 07-03-2013:
#	Added a version property and a related getter.
#	Added a .version command to the command line interface.
#	All commands now start with dot (e.g., .quit).
#	Added an .help command.
#	Added a welcome message.
#	Added support for local and remote usernames (the local one is sent to
#	the server, the remote one is fetched with every message and displayed
#	on the prompt).  Each user is given chance to change her name through
#	the command .username.  A pair of default names (LOCAL/REMOTE) are
#	given.


import json
import uuid


class Chat:

	__messageStack = []
	__clientId = None
	__lastReceivedMessageId = -1

	__localUsername = ""
	__remoteUsername = ""

	__actionWhenRemoteUsernameChanges = None

	__libraryVersion = "0.0.2"



	def __initClientId(self):
		self.__clientId = str(uuid.uuid4())

	def __prepareSendData(self):
		messages = []
		while self.countMessages() > 0:
			messages.append(self.popMessage())

		send_data = { 
			'client_id': self.__clientId,
			'username' :self.__localUsername,
			'messages': messages,
			'last_received': self.__lastReceivedMessageId
		}

		return json.dumps(send_data)
	
	def __sendData(self, send_data):
		# TODO: stub
		print send_data
		raw_response = None
		return raw_response
	
	def __decodeResponse(self, raw_response):
		if raw_response:
			response = json.loads(raw_response)
			rows = []
			for item in response:
				if self.__lastReceivedMessageId < int(item['id']):
					self.__lastReceivedMessageId = int(item['id'])
				if item['username'] != self.__remoteUsername:
					if self.__actionWhenRemoteUsernameChanges:
						self.__actionWhenRemoteUsernameChanges(item['username'])
				rows.append(item['body'])
			return rows

	def registerActionWhenRemoteUsernameChanges(self, action):
		self.__actionWhenRemoteUsernameChanges = action

	def popMessage(self):
		return self.__messageStack.pop()

	def pushMessage(self, message):
		self.__messageStack.append(message)
	
	def countMessages(self):
		return len(self.__messageStack)

	def sendMessages(self):
		send_data = self.__prepareSendData()
		raw_response = self.__sendData(send_data)
		return self.__decodeResponse(raw_response)
	
	def getRemoteUsername(self):
		return self.__remoteUsername

	def setUsername(self, username):
		self.__localUsername = username

	def getVersion(self):
		return self.__libraryVersion
	
	def __init__(self):
		self.__initClientId()


 
def remoteUsernameHasChanged(username):
	global remote_username
	print "%s> NOTICE: user has changed username to %s" % (remote_username, username)
	remote_username = username


chat = Chat()
chat.registerActionWhenRemoteUsernameChanges(remoteUsernameHasChanged)

version = chat.getVersion()
print "Welcome to gitalk version %s" % version
print "Commands begin with a dot.  Type '.help' to get help."

username = "LOCAL"
remote_username = "REMOTE"

while (True):
	message = raw_input("%s> " % username)
	if message == ".help":
		print "Available commands are: .help, .quit, .username, .version"
		continue
	if message == ".quit":
		exit()
	if message == ".username":
		username = raw_input("Set username: ")
		chat.setUsername(username)
		continue
	if message == ".version":
		print version
		continue

	chat.pushMessage(message)
	response = chat.sendMessages()
	if response:
		for line in response:
			print "%s> %s" % (remote_username, line)
